import java.net.*;
import java.io.*;
import java.util.*;

class Server implements Runnable {

  private static final int DEFAULT_PORT = 8765;
  private ServerSocket mainSock = null;
  private static Vector conVec = new Vector(5, 2); // connection vector

  public int this_port = DEFAULT_PORT;

  Thread runner;


  /**
   * constructor
   */
  public Server() {
     try {
        mainSock = new ServerSocket(this_port);
     } catch(Exception e) { 
        System.err.println("Can't create system socket, system exits."); 
        System.exit(1);
     }  
  }


  /**
   * waits for a connection then spawns a ServerManager to deal with it
   */
  public void run() {

     // continuously listen the socket
     while (true) {
        try {
           Socket this_connection = mainSock.accept();
	
           ServerManager manager = new ServerManager(this_connection, this);
           manager.start();
           conVec.addElement(manager);

           //clean up the vector if needed
           for (int i=0; i<conVec.size(); i++)
              if (!((ServerManager)(conVec.elementAt(i))).isAlive())
                  conVec.removeElementAt(i);
        } catch(Exception e) {
           System.err.println("Exception: " + e);
        }
     }
  }


  /**
   * start thread
   */   
  public void start() {
    if (runner == null); {
      runner = new Thread(this);
      runner.start();
    }
  }

  /**
   * stop thread
   */
  public void stop() {
    if (runner != null) {
      runner.stop();
      runner = null;
    }
  }


  public static void main(String[] args) {
     Server server = new Server();
     server.start();
  }

}


class ServerManager extends Thread {
   private Socket sock;
   private DataOutputStream out;
   private DataInputStream in;
   private PrintStream ps;

   private Server parent;

   /**
    * constructor
    * @param s server socket
    */
   public ServerManager(Socket s, Server parent) {
      sock = s;
      this.parent = parent;
   }


   /**
    * read commands from the client and do work
    */
   public void run() {
      try {
	out = new DataOutputStream(sock.getOutputStream());
	in  = new DataInputStream
	  (new BufferedInputStream(sock.getInputStream()));

        String hostname = sock.getInetAddress().getHostName();
        System.out.println("Connection from: " + hostname);
 

        doServerWork(hostname);
        sock.close();                   // close the socket

      } catch (Exception e) {
	e.printStackTrace();
      }
      stop();
   }

   ////////////////////////////////////////////////////
   // real worker
   // You need to write your codes in this function.
   // You don't need to change other parts until you know
   // know fully about all the codes.
   ///////////////////////////////////////////////////////
   private synchronized void doServerWork(String hostname) {
     
     File fl;
     FileOutputStream fout;
     PrintStream pout;
     String lNameTxt ,st;

     try {
       //System.out.println("Got from the client: " + in.readLine());
       st = in.readLine();
       System.out.println("Got from the client2: " + st);
       if(st.equals("nodeset.ini")) {
	 fout = new FileOutputStream("nodeset.ini");
	 pout = new PrintStream(fout);
	 while((st = in.readLine()) != null) {
	   pout.println(st);
	   pout.println(in.readLine());
	   pout.println("");
	 }
       }
       else if(st.equals("linkset.ini")) {
	 fout = new FileOutputStream("linkset.ini");
	 pout = new PrintStream(fout);
	 while((st = in.readLine()) != null) {
	   pout.println(st);
	   pout.println(in.readLine());
	   pout.println("");
	 } 
       }
       else if(st.equals("dataset.ini")) {
	   fout = new FileOutputStream("dataset.ini");
	   st = in.readLine();
	   //System.out.println("Got from the client2: " + st);
	   pout = new PrintStream(fout);
	   pout.print(st);
       }
       else if(st.equals("dist.ini")) {
	   fout = new FileOutputStream("dist.ini");
	   st = in.readLine();
	   //System.out.println("Got from the client2: " + st);
	   pout = new PrintStream(fout);
	   pout.print(st);
       }
       else if(st.equals("exec.script")) {
	   fout = new FileOutputStream("exec.scrpt");
	   pout = new PrintStream(fout);
	   while((st = in.readLine()) != null) {
	     //System.out.println("Got from the client2: " + st);
	     pout.println(st);
	     //pout.println("");
	   }
	 }
       else if(st.equals("run")) {
	 Process p  = Runtime.getRuntime().exec
	   ("./petasim_to_file");
	   //("/usr/bin/cp zxz xzx");
	 Process p2  = Runtime.getRuntime().exec
	   ("/usr/bin/cp results.out ../PetaSIM/V0.1/applet");

       }
       else if(st.equals("clearResult")) {

	 Process p  = Runtime.getRuntime().exec
	   ("/usr/bin/cp results.clean ../PetaSIM/V0.1/applet/results.out");
	 //("/usr/bin/cp results.clean results.out");
	 //System.out.println("Got from "+st);
	 Process p2  = Runtime.getRuntime().exec
	   ("/usr/bin/rm ./*.ini);
       }
     }
     catch (Exception e) {
       e.printStackTrace();
     }
   }
}
